#!/bin/sh -e
# --------
# File:        cddt.sh
# Description: Set of shell functions for the cddt scripts, the script
#              is executable to be loadable without hardcoding a PATH.
# Author:      Sergio Talens-Oliag <sto@debian.org>
# Copyright:   (c) 2005,2006 Sergio Talens-Oliag <sto@debian.org>
# SVN Id:      $Id: cddt.sh 392 2006-03-04 22:07:58Z sto $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along 
# with this program; if not, write to the Free Software Foundation, Inc., 
# 51 Franklin St, Fifth Floor, Boston MA 02110-1301 USA
# --------

# Load the ssft.sh code (if it is not available the program will fail)
. ssft.sh 

# ============ #
# L10N Support #
# ============ #

# Function: cddt_set_textdomain
# 
# Description: Saves the current TEXTDOMAIN and sets it to cddt-runtime

cddt_set_textdomain() {
  if [ "$TEXTDOMAIN" != "cddt-runtime" ]; then
    CDDT_OLD_TEXTDOMAIN="$TEXTDOMAIN"
    TEXTDOMAIN="cddt-runtime"
    export TEXTDOMAIN
  fi
}

# Function: cddt_reset_textdomain
# 
# Description: Resets the TEXTDOMAIN to its old value (if there is one)

cddt_reset_textdomain() {
  if [ -n "$CDDT_OLD_TEXTDOMAIN" ]; then
    TEXTDOMAIN="$CDDT_OLD_TEXTDOMAIN"
    export TEXTDOMAIN
    unset CDDT_OLD_TEXTDOMAIN
  fi
}

# ============ #
# Initial test #
# ============ #

# This script is primarily a shell function library, but it is put on the PATH
# to be able to load it using ". cddt.sh" (the idea was taken from the
# gettext.sh program).
#
# We use a function to test if the script has been called instead of sourced
# and in that later case we accept the --help and --version options.
# 
# The test is defined inside a function to be able to source the script from
# zsh, as this shell by default sets the option FUNCTION_ARGZERO that replaces
# the $0 variable for sourced scripts and functions; using a function we are
# sure that if FUNCTION_ARGZERO is set, $0 never takes the program name. Note
# that using this method if we execute the script using zsh the default
# options we will not see the --help or --version messages, but that is not
# important, as the executable script is run using '/bin/sh' and we get the
# desired efect when it is called directly.

cddt_sh_fhs_test() {
  # Function: cddt_sh_doc [FUNCTION_NAMES]
  # Description: prints the list of functions exported by the cddt.sh script
  # or shows the documentation related to it.
  cddt_sh_doc() {
    local cddt_sh="$0"
    if [ "$#" -lt 1 ]; then
      awk '/^# Function:/ { print $3 }' $cddt_sh
    else
      awk -v "name=$1" '/^# Function:/ { \
        if (($3 == name)) {              \
	  do {                           \
	    if (match($0, "^#")) {       \
	      print substr($0,3);        \
	    } else {                     \
	      printf "\n"; break;        \
	    }                            \
          } while (getline);             \
        }                                \
      }' $cddt_sh
    fi
  }
  
  # Function: cddt_sh_usage
  # Description: outputs to stdout the --help usage message.
  cddt_sh_usage() {
    cddt_set_textdomain
    local MSG=$(eval_gettext \
"CDDT Shell script function library (version \$version)
Usage: . cddt.sh

When called directly the program supports the following options:

 -d,--doc [FUNCTIONS]  Prints the list of available functions. If function
                       names are given prints functions' documentation.
 -h,--help             This message
 -v,--version          File version
")
    cddt_reset_textdomain
    echo "$MSG"
  }
  
  # Function: cddt_sh_version
  # Description: outputs to stdout the --version message.
  cddt_sh_version() {
    cddt_set_textdomain
    local MSG="`eval_gettext \
"Copyright (C) 2006 Sergio Talens-Oliag <sto@debian.org>
This is free software; see the source for copying conditions.  There is NO
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."`"
    ssft_reset_textdomain
    echo "$progname ($package $version)"
    echo "$MSG"
  }
  
  case "$0" in
    cddt.sh | */cddt.sh | *\cddt.sh)
      progname=$0
      package=cddt
      version="_@_CDDT_VERSION_@_"
      if test $# -ge 1; then
        case "$1" in
	  --doc | --do | --d | -d)
	    shift; cddt_sh_doc "$@"; exit 0 ;;
          --help | --hel | --he | --h | -h)
            cddt_sh_usage; exit 0 ;;
          --version | --versio | --versi | --vers | --ver | --ve | --v | -v)
            cddt_sh_version; exit 0 ;;
        esac
      fi
      cddt_sh_usage 1>&2
      exit 1
    ;;
  esac
  # unset auxiliary functions
  unset -f cddt_sh_doc
  unset -f cddt_sh_usage
  unset -f cddt_sh_version
}

# Execute the test function
cddt_sh_fhs_test "$@"

# Unset the test function (it is no longer needed)
unset -f cddt_sh_fhs_test

#############
# FUNCTIONS #
#############

# ============== #
# Initialization #
# ============== #

# Function: cddt_initialize
#
# Description: Function to initizalize the LLXCFG scripts, it sets the default
# PATH, adjusts some variables to their default values and sources the default
# configuration file.
# Note the the default variable values can be modified simply by setting the
# value on the default configuration file.

cddt_initialize() {
  # Default configuration directory
  [ -z "$CDDT_CONFDIR" ] && \
  CDDT_CONFDIR="/etc/cddt"
  
  # Default configuration file
  [ -z "$CDDT_CONFFILE" ] && \
  CDDT_CONFILE="$CDDT_CONFDIR/cddt.conf"
  
  # Default cddt script directories
  [ -z "$CDDT_APT_AVAIL_SCRIPTS" ] && \
  CDDT_APT_AVAIL_SCRIPTS="$CDDT_CONFDIR/pkg-scripts"
  [ -z "$CDDT_APT_ENABL_SCRIPTS" ] && \
  CDDT_APT_ENABL_SCRIPTS="$CDDT_CONFDIR/pkg-scripts.d"
  [ -z "$CDDT_NET_AVAIL_SCRIPTS" ] && \
  CDDT_NET_AVAIL_SCRIPTS="$CDDT_CONFDIR/net-scripts"
  [ -z "$CDDT_NET_ENABL_SCRIPTS" ] && \
  CDDT_NET_ENABL_SCRIPTS="$CDDT_CONFDIR/net-scripts.d"
  
  # Default cddt template directories
  [ -z "$CDDT_TMPLDIR" ] && \
  CDDT_TMPLDIR="/usr/share/cddt/tmpl"

  # Default cddt variable directories
  if [ -z "$CDDT_VARFILE_DIRS" ]; then
    CDDT_VARFILE_DIRS="/usr/share/cddt/vars"
    CDDT_VARFILE_DIRS="$CDDT_VARFILE_DIRS /var/lib/cddt/vars"
    CDDT_VARFILE_DIRS="$CDDT_VARFILE_DIRS /etc/cddt/vars"
  fi

  # Save the environment SSFT_FRONTEND value
  local env_frontend="$SSFT_FRONTEND"

  # Source the default configuration file
  if [ -f "$CDDT_CONFILE" ]; then
    source "$CDDT_CONFILE"
  fi

  # Reset the SSFT_FRONTEND environment value if needed (the environment
  # value has more precedence than the config file value).
  if [ -n "$env_frontend" ]; then
    SSFT_FRONTEND="$env_frontend";
  fi

  # If it is not set, use the default frontend (zenity, dialog or text)
  if [ -z "$SSFT_FRONTEND" ]; then
    SSFT_FRONTEND="`ssft_choose_frontend`"
  fi
}

# =========================== #
# FILE MANIPULATION FUNCTIONS #
# =========================== #

# Function: cddt_file_install [[OWNER:][MODE]] ORIG_FILE DEST_FILE
#
# Description: Install the ORIG_FILE over DEST_FILE using the cddt-divert
# script to do it. The installed file gets the OWNER and MODE given as
# arguments. If owner is not given, nobody owns the file; if MODE is omited,
# 0755 is used.

cddt_file_install(){
  local mode
  local orig
  local dest
  local status
  if [ "$#" -eq "2" ]; then
    mode="0755"
    orig="$1"
    dest="$2"
  elif [ "$#" -eq "3" ]; then
    owner="`echo "$1" | cut -d: -f1`"
    if [ "$owner" = "$1" ]; then
      owner=""
      mode="$1"
    else
      mode="`echo "$1" | cut -d: -f2`"
      if [ -z "$mode" ]; then
        mode="0755"
      fi
    fi
    orig="$2"
    dest="$3"
  else
    return 1
  fi
  # Remove diversion before updating
  status="`cddt-divert status $dest`" || return 1
  if [ -n "$status" ]; then
    cddt-divert del --force $owner $dest || return 1
  fi
  # Change "orig" permissions to make sure that the repl copy gets the right
  # permissions
  chmod "$mode" "$orig" || return 1
  # Add new diversion
  cddt-divert add $owner "$dest" "$orig" || return 1
  # And set mode on "dest", just in case it's not right
  chmod "$mode" "$dest" || return 1
  return 0
}

# Function: cddt_file_check FILES
#
# Description: Validate that the files passed as argument are available

cddt_file_check() {
  # Local variables
  local f
  local ret
  local TIT
  local MSG
  local MISSING_FILES
  # Check arguments
  if [ "$#" -lt "1" ]; then
    return 1
  fi
  # Check files
  ret=0
  MISSING_FILES=""
  for f in "$@"; do
    if [ ! -f "$f" ]; then
       if [ -z "$MISSING_FILES" ]; then
         MISSING_FILES="- '$f'"
       else
         MISSING_FILES="$MISSING_FILES
- '$f'"
       fi
       ret=1
    fi
  done
  if [ $ret -ne 0 ]; then
    cddt_set_textdomain
    TIT="`gettext "Required files missing"`"
    MSG=$(eval_gettext "The following files are missing:

\$MISSING_FILES

Check your installation")
    cddt_reset_textdomain
    ssft_display_error "$TIT" "$MSG"
  fi
  return $ret;
}

# ========================= #
# NETWORK RELATED FUNCTIONS #
# ========================= #

# Function: cddt_net_ifup_check IFACE
#
# Description: Return 0 if a network interface has been brought up by debian's
# ifupdown scripts or 1 otherwise

cddt_net_ifup_check() {
  local IFSTATE_FILE="/etc/network/run/ifstate"
  if grep -q "^$1=" "$IFSTATE_FILE"; then
    return 0;
  else
    return 1; 
  fi
}

# Function: cddt_net_show_IPADDR IFACE
#
# Description: Print the configured IP address of the given interface

cddt_net_show_IPADDR() {
  local IFCONFIG="`which ifconfig`"
  if [ -n "$1" -a -x "$IFCONFIG" ]; then
    $IFCONFIG "$1" | sed -n -e '/^[[:blank:]]*inet addr:/ {
      s/[[:blank:]]\+/ /g;
      s/^ inet addr:\(.*\) Bcast:\(.*\) Mask:\(.*\)$/\1/;
      p;
    }' 2> /dev/null
  fi
}

# Function: cddt_net_show_IPBCAST IFACE
#
# Description: Print the configured broadcast address of the given interface

cddt_net_show_IPBCAST() {
  local IFCONFIG="`which ifconfig`"
  if [ -n "$1" -a -x "$IFCONFIG" ]; then
    $IFCONFIG "$1" | sed -n -e '/^[[:blank:]]*inet addr:/ {
      s/[[:blank:]]\+/ /g;
      s/^ inet addr:\(.*\) Bcast:\(.*\) Mask:\(.*\)$/\2/;
      p;
    }' 2> /dev/null
  fi
}

# Function: cddt_net_show_IPMASK IFACE
#
# Description: Print the configured network mask of the given IFACE

cddt_net_show_IPMASK() {
  local IFCONFIG="`which ifconfig`"
  if [ -n "$1" -a -x "$IFCONFIG" ]; then
    $IFCONFIG "$1" | sed -n -e '/^[[:blank:]]*inet addr:/ {
      s/[[:blank:]]\+/ /g;
      s/^ inet addr:\(.*\) Bcast:\(.*\) Mask:\(.*\)$/\3/;
      p;
    }' 2> /dev/null
  fi
}

# ====================== #
# VARIABLE SUBSTITUTIONS #
# ====================== #

# Function: cddt_list_files DIRECTORIES
#
# Description: Print the list of files contained on the directories passed as
# arguments

cddt_list_files() {
  local varfiles=""
  for d in "$@"; do
    test -d "$d" && find "$d" -maxdepth 1 -type f -exec echo \"{}\" \;
  done
}

# Function: cddt_find_replvars LIST_OF_FILENAMES
# 
# Description: Given a set of files as arguments, show all the replaceable
# variables of the form _@_VARIABLE_@_ contained on them.

cddt_find_replvars() {
  if [ "$#" -gt "0" ]; then
    awk -F'_@_' '{ for(n=2;n<NF;n+=2) { print $n } }' "$@" \
    | grep "^[[:alpha:]_][[:alnum:]_]*$" | sort -u
  fi
}

# Function: cddt_repl_stdin_envvars LIST_OF_VARIABLES < TEXT_TO_REPLACE
#
# Description: Given a set of environment variables as arguments, replace the
# ocurrences of the strings _@_VARIABLE_@_ with the variable value. Note that
# the replacement is done against the standard input and the replacement is
# done on the standard output. Note that if the VARIABLE is not defined it is
# replaced by the empty string

# The SEDCMD is built using the echo command, but the value is first escaped
# (existing backslashes and % are prefixed with a backslash)

cddt_repl_stdin_envvars() {
  local varsed='s/\\/\\\\/g;s/%/\\%/g;';
  local SEDCMD="$(
    for v in "$@"; do \
      val="`eval echo \\\${$v} | sed -e "$varsed"`"; \
      echo -n "s%_@_${v}_@_%${val}%g;"; \
    done \
  )"
  sed -e "$SEDCMD"
}

# Function: cddt_repl_stdin_fvars LIST_OF_VFILES < TEXT_TO_REPLACE
#
# Description: Given a set of files as arguments, read all the variables
# defined on them and replace the ocurrences of the strings _@_VARIABLE_@_ with
# the variable value. Note that the replacement is done against the standard
# input and the replacement is done on the standard output.

# The SEDCMD is built using one script to escape the value (existing
# backslashes and % are prefixed with a backslash) and the second sed does the
# actual replacement. The split has been done to avoid problems with the shell
# escape mechanisms.

cddt_repl_stdin_fvars() {
  local SEDCMD="$(
    eval cddt_show_filevars "$@" \
    | sed -e 's/\\/\\\\/g;s/%/\\%/g;' \
    | sed -n -e "/^.*='.*';$/ { s/^\(.*\)='\(.*\)';/s%_@_\1_@_%\2%g;/; p }" \
  )"
  sed -e "$SEDCMD"
}

# Function: cddt_repl_file_envvars FILENAME > REPL_TEXT
# 
# Description: Given a file as argument, replace all the variables it has
# getting them from the environment. The replaced text is shown on the
# standard output.

cddt_repl_file_envvars(){
  local filename="$1"
  local variables=""
  if [ -r "$filename" ]; then
    variables=$(cddt_find_replvars "$filename") 
    eval cddt_repl_stdin_envvars `echo "$variables"` < "$filename"
  fi
}

# Function: cddt_repl_file_fvars FILENAME LIST_OF_VFILES > REPL_TEXT
# 
# Description: Given a file as argument, replace all the variables contained
# on the file getting them from the LIST_OF_VFILES. If the variable is not
# found it is replaced by the empty string. The replaced text is shown on the
# standard output.

cddt_repl_file_fvars(){
  local filename="$1"
  if [ -r "$filename" ]; then
    shift
    cddt_repl_stdin_fvars "$@" < "$filename" \
    | sed -e 's/_@_[[:alpha:]_][[:alnum:]_]*_@_//g';
  fi
}

# Function: cddt_show_filevars LIST_OF_VFILES
#
# Description: Given a set of files as arguments, show all the variables
# defined on them, using always the last value (if a variable is redefined
# its value is overwritten).

# The function uses sed to get only lines on the correct format and remove
# whitespace, double quotes around the value and all the quotes and to put
# quotes around the given value.
#
# Note that if the value has a ; at the end, it is accepted if the variable
# value is written between " or '.
#
# After the sed processing awk is used to print only the last value of the
# variable

cddt_show_filevars() {
  if [ "$#" -gt 0 ]; then
    eval cat `echo $@` 2> /dev/null \
    | sed -n -e "\
    /^[[:blank:]]*[[:alpha:]_][[:alnum:]_]*[[:blank:]]*=/ {\
      s%^[[:blank:]]*%%;\
      s%[[:blank:]]*$%%;\
      s%[[:blank:]]*=[[:blank:]]*%=%;\
      s%='\(.*\)';*$%='\1'%;\
      s%=\"\(.*\)\";*$%=\1%;\
      s%'%%g;\
      s%=\(.*\)$%='\1';%;\
      p;\
    };" 2> /dev/null \
    | awk -F'=' -- '{ v[$1] = $0; } END { for (k in v) print v[k]; }'
  fi
}

# Function: cddt_showvars LIST_OF_VARIABLES
#
# Description: Simple script to read configuration variables, it reads the
# files referenced by CDDT_VARFILE_DIRS and prints the values of the
# variables passed as arguments to the function on a format suitable
# for source them from a shell script

cddt_showvars() {
  # Calculate the expression to print only the files we want
  local PARAMS_REGEX=""
  local varfiles=""
  for v in "$@"; do
    if [ -z "$PARAMS_REGEX" ]; then
      PARAMS_REGEX="^$v=";
    else
      PARAMS_REGEX="$PARAMS_REGEX\|^$v="
    fi
  done
  if [ -n "$PARAMS_REGEX" ]; then
    # Get all variables from the system varfiles and print only the ones we want
    varfiles=$(eval cddt_list_files `echo $CDDT_VARFILE_DIRS`)
    cddt_show_filevars $varfiles | grep "$PARAMS_REGEX"
  fi
}

# ========================== #
# TEMPLATE FILE MANIPULATION #
# ========================== #

# Function: cddt_tmpl_check TMPL_DIR TMPL_FILES
#
# Description: Validate that the needed templates are available

cddt_tmpl_check() {
  # Local variables
  local f
  local ret
  local TIT
  local MSG
  local MISSING_FILES
  local TMPL_DIR
  # Check arguments
  if [ "$#" -lt "2" ]; then
    return 1
  fi
  # Set arguments
  TMPL_DIR="$1"
  shift
  # Check files
  ret=0
  MISSING_FILES=""
  for f in "$@"; do
    if [ ! -f "$TMPL_DIR/$f" ]; then
       if [ -z "$MISSING_FILES" ]; then
         MISSING_FILES="- '${TMPL_DIR}${f}'"
       else
         MISSING_FILES="$MISSING_FILES
- '${TMPL_DIR}${f}'"
       fi
       ret=1
    fi
  done
  if [ $ret -ne 0 ]; then
    cddt_set_textdomain
    TIT="`gettext "Missing template files"`"
    MSG=$(eval_gettext "The following templates are missing:

\$MISSING_FILES

Check your installation")
    cddt_reset_textdomain
    ssft_display_error "$TIT" "$MSG"
  fi
  return $ret;
}

# Function: cddt_tmpl_install OWNER TMPL_DIR TEMPLATES
#
# Description: Generates and installs the template files using the environment
# variables as input. All the installed templates belong to the given OWNER,
# if the installation must be anonymous use the empty string ('') as OWNER.

cddt_tmpl_install(){
  local TIT
  local EMSG
  local TMPLFILE
  local ret
  # Check arguments
  if [ "$#" -lt "3" ]; then
    return 1
  fi
  # Set arguments
  OWNER="$1:"
  TMPL_DIR="$2"
  shift 2
  # Create temporary file
  TMPFILE="`tempfile -p tmpl_ -s _cddt_smbldap`"
  # Install templates
  for f in "$@"; do
    ret=0
    EMSG="$( (
      cddt_repl_file_envvars "$TMPL_DIR/$f" > "$TMPFILE";
      cddt_file_install "${OWNER}0644" "$TMPFILE" "$f";
    ) 2>&1 )" || ret=$?
    if [ "$ret" -ne "0" ]; then
      cddt_set_textdomain
      TIT="`gettext "Could not install configuration file"`"
      cddt_reset_textdomain
      ssft_display_error "$TIT" "Error: $EMSG"
      rm -f "$TMPFILE"
      return $ret
    fi
  done
  rm -f "$TMPFILE"
  return 0
}  

# Function: cddt_tmpl_list TEMPLATES
#
# Description: Print the list of templates, one per line

cddt_tmpl_list() {
  for f in "$@"; do
    echo $f;
  done
}

# Function: cddt_tmpl_remove OWNER TEMPLATES
#
# Description: Remove template diversions owned by OWNER (use the empty string
# as OWNER to remove any template).

cddt_tmpl_remove(){
  local TIT
  local EMSG
  local owner
  local ret
  local status
  # Check arguments
  if [ "$#" -lt "2" ]; then
    return 1
  fi
  owner="$1"
  shift
  # Remove templates
  for f in $@; do
    ret=0
    EMSG=$( (
      if [ -n "`cddt-divert status "$f"`" ]; then
        cddt-divert del --force $owner $f
      fi
    ) 2>&1 ) || ret=$?
    if [ "$ret" -ne "0" ]; then
      cddt_set_textdomain
      TIT="`gettext "Error removing diversion"`"
      cddt_reset_textdomain
      ssft_display_error "$TIT" "Error: $EMSG"
      return $ret
    fi
  done
  return 0
}

# Function: cddt_tmpl_status TEMPLATES
#
# Description: Print status, can be one of 'broken', 'disabled', 'enabled' or
# 'modified'.

cddt_tmpl_status(){
  # Local variables
  local disabled=false
  local diverted=false
  local modified=false
  # Check arguments
  if [ "$#" -lt "1" ]; then
    return 1
  fi
  # Check files
  for f in $@; do
    case "`cddt-divert status "$f"`" in
      "")       disabled=true;;
      diverted) diverted=true;;
      modified) modified=true;;
      *);;
    esac
  done
  if $disabled && $diverted || $modified; then
    echo "broken"
  elif $disabled; then
    echo "disabled"
  elif $modified; then
    echo "modified"
  else
    echo "enabled"
  fi
  return 1
}  

# ===================== #
# SCRIPT DIRS FUNCTIONS #
# ===================== #

# Function: cddt_scripts_list SCRIPTS_DIR
#
# Description: Print the names of the valid scripts of the given directory.
# Can be used to see the list of available or enabled scripts.

cddt_scripts_list() {
  test -d "$1" || return 0
  for s in `run-parts --test "$1" 2> /dev/null`; do
    basename "$s";
  done
}

# Function: cddt_scripts_disable ORIG_DIR DEST_DIR SCRIPT_NAMES
#
# Description: Remove links to the given script names on the DEST_DIR. The
# function fails if we try to remove a script that is not a symlink or does
# not point to the ORIG_DIR script (it does not need to exist, but the link
# has to point to it). The script ignores already disabled scripts.

cddt_scripts_disable() {
  # Local vars
  local ORIG_DIR
  local DEST_DIR
  local ERR_TEXT
  local err
  local errmsg
  local dest_scr
  local orig_scr
  local hide_scr 
  local disable_list
  # Check params
  test "$#" -gt "2" || return 1
  # Check dirs
  ORIG_DIR="$1"
  DEST_DIR="$2"
  test -d "$ORIG_DIR" -a -d "$DEST_DIR" || return 1
  shift 2
  # Compute the list of scripts to enable
  disable_list=""
  ERR_TEXT=""
  for s in "$@"; do
    errmsg=""
    # Compute filenames
    dest_scr="$DEST_DIR/$s"
    [ -L "$dest_scr" ] && dest_scr="`readlink -f "$DEST_DIR/$s" 2> /dev/null`"
    orig_scr="$ORIG_DIR/$s"
    [ -L "$orig_scr" ] && orig_scr="`readlink -f "$ORIG_DIR/$s" 2> /dev/null`"
    hide_scr="`ls "$DEST_DIR/.$s" 2> /dev/null`" || true
    # Decide what to do with the script
    if [ -n "$hide_scr" ]; then
      # Continue if it is hidden
      continue
    elif [ "$orig_scr" != "$dest_scr" ]; then
      # Print error if it does not point to the right place
      cddt_set_textdomain
      errmsg=$(eval_gettext "'\$s' is not a link to the right script")
      cddt_reset_textdomain
    else
      # The script points to the right place
      disable_list="$disable_list $s"
    fi
    if [ -z "$ERR_TEXT" ]; then
      ERR_TEXT="$errmsg"
    else
      ERR_TEXT="$ERR_TEXT
$errmsg"
    fi
  done
  # Disable scripts
  if [ -z "$ERR_TEXT" ]; then
    for s in $disable_list; do
      err=$(rm -f "$DEST_DIR/$s" 2>&1);
      if [ -n "$err" ]; then
        cddt_set_textdomain
        errmsg=$(eval_gettext "Error disabling script '\$s': \$err")
        cddt_reset_textdomain
      fi 
      if [ -z "$ERR_TEXT" ]; then
        ERR_TEXT="$errmsg"
      else
        ERR_TEXT="$ERR_TEXT
$errmsg"
      fi
    done
  fi
  # Print error messages if any
  if [ -n "$ERR_TEXT" ]; then
    cddt_set_textdomain
    ERR_TITLE="`gettext "Errors disabling scripts"`"
    cddt_reset_textdomain
    ssft_display_error "$ERR_TITLE" "$ERR_TEXT"
    return 1
  fi
  return 0
}  

# Function: cddt_scripts_enable ORIG_DIR DEST_DIR SCRIPT_NAMES
# 
# Description: Create links to the given script names on the DEST_DIR. The
# command fails and does nothing if any of the SCRIPT_NAMES is not available,
# but silently ignores scripts hidden by the user or already enabled.

cddt_scripts_enable() {
  # Local vars
  local ORIG_DIR
  local DEST_DIR
  local ERR_TEXT
  local err
  local errmsg
  local dest_scr
  local orig_scr
  local hide_scr
  local enable_list
  # Check params
  test "$#" -gt "2" || return 1
  # Check dirs
  ORIG_DIR="$1"
  DEST_DIR="$2"
  test -d "$ORIG_DIR" -a -d "$DEST_DIR" || return 1
  shift 2
  # Compute the list of scripts to enable
  enable_list=""
  ERR_TEXT=""
  for s in "$@"; do
    errmsg=""
    # First check if the script is available
    available="false"
    for es in `cddt_scripts_list "$ORIG_DIR"`; do
      if [ "$es" = "$s" ]; then
        available="true"; break;
      fi
    done
    if $available; then 
      # The script is available, don't touch it if it is hidden or points to the right place.
      dest_scr="$DEST_DIR/$s"
      [ -L "$dest_scr" ] && dest_scr="`readlink -f "$DEST_DIR/$s" 2> /dev/null`"
      orig_scr="$ORIG_DIR/$s"
      [ -L "$orig_scr" ] && orig_scr="`readlink -f "$ORIG_DIR/$s" 2> /dev/null`"
      hide_scr="`ls "$DEST_DIR/.$s" 2> /dev/null`" || true
      if [ -n "$hide_scr" -o "$orig_scr" = "$dest_scr" ]; then
        continue
      fi
      # We know that dest_scr does not point to the right place, now check if it
      # exists or not)
      if [ -n "`ls "$DEST_DIR/$s" 2> /dev/null`" ]; then
        cddt_set_textdomain
        errmsg=$(eval_gettext "'\$s' is not a link to the right script")
        cddt_reset_textdomain
      else
        enable_list="$enable_list $s"
      fi
    else
      # The script is missing
      cddt_set_textdomain
      errmsg=$(eval_gettext "The script '\$s' does not exist or can't be run")
      cddt_set_textdomain
    fi
    if [ -z "$ERR_TEXT" ]; then
      ERR_TEXT="$errmsg"
    else
      ERR_TEXT="$ERR_TEXT
$errmsg"
    fi
  done
  # Enable scripts
  if [ -z "$ERR_TEXT" ]; then
    for s in $enable_list; do
      err=$(ln -sf "$ORIG_DIR/$s" "$DEST_DIR/$s" 2>&1);
      if [ -n "$err" ]; then
        cddt_set_textdomain
        errmsg=$(eval_gettext "Error enabling script '\$s': \$err")
        cddt_reset_textdomain
      fi 
      if [ -z "$ERR_TEXT" ]; then
        ERR_TEXT="$errmsg"
      else
        ERR_TEXT="$ERR_TEXT
$errmsg"
      fi
    done
  fi
  # Print error messages if any
  if [ -n "$ERR_TEXT" ]; then
    cddt_set_textdomain
    ERR_TITLE="`gettext "Errors enabling scripts"`"
    cddt_reset_textdomain
    ssft_display_error "$ERR_TITLE" "$ERR_TEXT"
    return 1
  fi
  return 0
}  

# Function: cddt_scripts_hide ORIG_DIR DEST_DIR SCRIPT_NAMES
# 
# Description: Create hidden links to the given script names on DEST_DIR.
# With those links the scripts are ignored if available. The function creates
# the links without checking if the scripts are available to be able to ignore
# something before installing it.

cddt_scripts_hide() {
  # Local vars
  local ORIG_DIR
  local DEST_DIR
  local ERR_TEXT
  local err
  local errmsg
  # Check params
  test "$#" -gt "2" || return 1
  # Check dirs
  ORIG_DIR="$1"
  DEST_DIR="$2"
  test -d "$ORIG_DIR" -a -d "$DEST_DIR" || return 1
  shift 2
  # Hide scripts
  for s in $@; do
    err=$(ln -sf "$ORIG_DIR/$s" "$DEST_DIR/.$s" 2>&1);
    if [ -n "$err" ]; then
      cddt_set_textdomain
      errmsg=$(eval_gettext "Error hidding script '\$s': \$err")
      cddt_reset_textdomain
    fi 
    if [ -z "$ERR_TEXT" ]; then
      ERR_TEXT="$errmsg"
    else
      ERR_TEXT="$ERR_TEXT
$errmsg"
    fi
  done
  # Print error messages if any
  if [ -n "$ERR_TEXT" ]; then
    cddt_set_textdomain
    ERR_TITLE="`gettext "Errors hidding scripts"`"
    cddt_reset_textdomain
    ssft_display_error "$ERR_TITLE" "$ERR_TEXT"
    return 1
  fi
  return 0
}  

# Function: cddt_scripts_show ORIG_DIR DEST_DIR SCRIPT_NAMES
# 
# Description: Remove hidden links to the given script names on DEST_DIR.
# Without those links the enable/disable functions work as expected.

cddt_scripts_show() {
  # Local vars
  local ORIG_DIR
  local DEST_DIR
  local ERR_TEXT
  local err
  local errmsg
  # Check params
  test "$#" -gt "2" || return 1
  # Check dirs
  ORIG_DIR="$1"
  DEST_DIR="$2"
  test -d "$ORIG_DIR" -a -d "$DEST_DIR" || return 1
  shift 2
  # Show scripts
  for s in $@; do
    err=$(rm -f "$DEST_DIR/.$s" 2>&1);
    if [ -n "$err" ]; then
      cddt_set_textdomain
      errmsg=$(eval_gettext "Error showing script '\$s': \$err")
      cddt_reset_textdomain
    fi 
    if [ -z "$ERR_TEXT" ]; then
      ERR_TEXT="$errmsg"
    else
      ERR_TEXT="$ERR_TEXT
$errmsg"
    fi
  done
  # Print error messages if any
  if [ -n "$ERR_TEXT" ]; then
    cddt_set_textdomain
    ERR_TITLE="`gettext "Errors showing scripts"`"
    cddt_reset_textdomain
    ssft_display_error "$ERR_TITLE" "$ERR_TEXT"
    return 1
  fi
  return 0
}  

# Function: cddt_scripts_status ORIG_DIR DEST_DIR SCRIPT_NAMES
# 
# Description: Show the status of the script names passed as argument, one per
# line.
#
# The output format is 'STATUS SCRIPT_NAME' where STATUS is one of:
#
# - disabled: the script is available but not enabled
# - disabled (hidden): the script is available but not enabled and its status
#   can't change
# - enabled: the script is available and enabled
# - enabled (hidden): the script is available and enabled and its status 
#   can't change
# - hidden: the script is not available and its status can't change
# - installed: there is a script installed on DEST_DIR, but it is not linked
#   to the one on ORIG_DIR
# - missing: the script is not available or installed

cddt_scripts_status() {
  # Local vars
  local ORIG_DIR
  local DEST_DIR
  local ERR_TEXT
  local err
  local errmsg
  local dest_scr
  local orig_scr
  # Check params
  test "$#" -gt "2" || return 1
  # Check dirs
  ORIG_DIR="$1"
  DEST_DIR="$2"
  test -d "$ORIG_DIR" -a -d "$DEST_DIR" || return 1
  shift 2
  # Test the status of the scripts
  ERR_TEXT=""
  for s in "$@"; do
    errmsg=""
    # Check if the script is enabled
    enabled="false"
    for es in `cddt_scripts_list "$DEST_DIR"`; do
      if [ "$es" = "$s" ]; then
        enabled="true"; break;
      fi
    done
    # Check if the script is available or not
    available="false"
    for as in `cddt_scripts_list "$ORIG_DIR"`; do
      if [ "$as" = "$s" ]; then
        available="true"; break;
      fi
    done
    # Check if the script is hidden
    if [ -n "`ls "$DEST_DIR/.$s" 2> /dev/null`" ]; then
      hidden="true"
    else
      hidden="false"
    fi
    # Now check the script links
    dest_scr="$DEST_DIR/$s"
    [ -L "$dest_scr" ] && dest_scr="`readlink -f "$DEST_DIR/$s" 2> /dev/null`"
    orig_scr="$ORIG_DIR/$s"
    [ -L "$orig_scr" ] && orig_scr="`readlink -f "$ORIG_DIR/$s" 2> /dev/null`"
    # Print status
    if $enabled; then
      if [ "$dest_scr" != "$orig_scr" ]; then
        echo "$s installed" 
      elif $hidden; then
        echo "$s enabled (hidden)"
      else
        echo "$s enabled"
      fi
    elif $available; then
      if $hidden; then
        echo "$s disabled (hidden)"
      else
        echo "$s disabled"
      fi
    elif $hidden; then
      echo "$s hidden"
    else
      echo "$s missing"
    fi 
  done
}  

# Function: cddt_scripts_status_avail ORIG_DIR DEST_DIR
# 
# Description: Show the status of the scripts found on ORIG_DIR

cddt_scripts_status_avail() {
  local ret=0
  cddt_scripts_status "$1" "$2" `cddt_list_scripts "$1"` || ret=$?
  return $ret
}

# ------
# SVN Id: $Id: cddt.sh 392 2006-03-04 22:07:58Z sto $
